<?php

namespace Intergo\Providers\MessageManager\Drivers;

use Intergo\Providers\Enums\FailedReasonEnum;
use Intergo\Providers\Exceptions\CustomException;
use Intergo\Helpers\HashHelper;
use GuzzleHttp\Client;

class AlarisDriver extends Driver
{
    protected $provider;

    /**
     * @var
     */
    protected $path;

    /**
     * Priority of the message
     *
     * @var int $priority
     */
    protected $priority = 2;

    /**
     * Send messages
     *

     * @return array
     */
    public function send()
    {
        // Format 1 => MULTI MESSAGE ==> [[to: "+9779856034616", message: "This is test"], [to: "+9779856034617", message: "This is test"]]
        if (!empty($this->messages))
        {
            return $this->sendSmsToRecipient($this->messages);
        }

        // Format 3 => CAMPAIGN MESSAGE ==> to: ["+9779856034616", "+9779856034617"], message: "This is test"
        return $this->sendSmsToRecipient($this->prepareData($this->recipient, $this->message));
    }

    /**
     * Prepare common data
     *

     * @param array $recipients
     * @param string $message
     * @return array
     */
    public function prepareData(array $recipients, string $message)
    {
        $messages = [];
        foreach ($recipients as $phone) {
            $messages[] = ['phone' => $phone, 'text' => $message, '_id' => HashHelper::generateUniqueID()];
        }
        return $messages;
    }

    /**
     * Prepare and send messages
     *

     * @param array $messages
     * @return type
     */
    public function sendSmsToRecipient(array $messages)
    {
        $response = [];
        foreach ($messages as $message) {
            $data = $this->getProviderData($message['provider']);
            $data['dnis'] = str_replace('+', '', $message['to']);
            $data['message'] = $message['message'];
            $data['flash'] = ($this->isFlashSMS) ? 1 : 0;

            if (!empty($this->sender))
            {
                $data['ani'] = $this->sender;
            }
            $providerResponse = $this->sendRequest($this->path, $this->provider->method, $data);
            if (!isset($message['_id']))
            {
                $response['FAILED'][] = $providerResponse;
            } else
            {
                $providerResponse['_id'] = $message['_id'];
                $response[$providerResponse['status']][] = $providerResponse;
            }
        }
        return $response;
    }

    /**
     * Get provider data
     *

     * @return array
     */
    public function getProviderData($provider)
    {
        $this->provider = $provider;

        //http://62.67.222.162:8001/api?username=SMStoOTP&password=password&dnis=35799123456&message=Thisisatest&command=submit&longMessageMode=split_or_payload&flash=1
        $ip = $this->provider->ip;
        $port = $this->provider->port;
        $password = $this->provider->password;
        $username = $this->provider->username;
        $this->path = "http://$ip:$port/api";
        if (!empty($this->provider->url))
        {
            $this->path = $this->provider->url;
        }

        $data = [
            'username' => $username,
            'password' => $password,
            'longMessageMode' => 'split_or_payload',
            'command' => 'submit'
        ];

        if ($this->provider->callback_url)
        {
            $data['dlr-url'] = $this->provider->callback_url;
        }

        return $data;
    }

    /**
     * Guzzle HTTP Request for all API Call
     *

     * @param       $path
     * @param       $method
     * @param array $data
     *
     * @return array
     */
    public function sendRequest($path, $method, $data = [])
    {
        $client = new Client();
        try
        {
            $resp = $client->post($path, [
                'form_params' => $data,
            ])->getBody()->getContents();
            $response = json_decode($resp, true);

            if (isset($response[$this->getMessageField()]))
            {
                $messageId = $response[$this->getMessageField()];
                if (empty($messageId))
                {
                    $response = [
                        'error' => true,
                        'reason' => 'Cannot send to Alaris Provider (empty message): ' . $resp,
                        'data' => $data,
                        'status' => 'REJECTED', // Initial rejection
                    ];
                } else
                {
                    $response = [
                        'message_id' => $messageId,
                        'status' => 'SENT-TO-PROVIDER',
                    ];
                }
            } elseif(isset($response[0][$this->getMessageField()])){

                $id0 = $response[0][$this->getMessageField()];
                $response = [
                    'message_id' => $id0,
                    'status' => 'SENT-TO-PROVIDER',
                ];
            }else
            {
                $response = [
                    'error' => true,
                    'reason' => 'Cannot send to Alaris Provider (No phone in response): ' . $resp,
                    'data' => $data,
                    'status' => 'REJECTED',
                ];
            }
        } catch (\Exception $e)
        {
            $response = [
                'error' => true,
                'reason' => HashHelper::maskValue($data['password'] ?? '', $e->getMessage()),
                'data' => $data,
                'status' => 'REJECTED',
            ];
        }
        return $response;
    }

    /**
     * After sending message provider(provider have different fields in response), I want to map different responses to have standard mapping
     * With this function, I will be getting standardized fields for all providers
     *

     * @param array $resultMessage
     * @return array
     */
    public function mapper(array $resultMessage)
    {
        if (!isset($resultMessage[$this->getMessageField()], $resultMessage['state']))
        {
            throw new CustomException('Invalid format', 400);
        }

        $resultMessage['provider'] = 'Alaris';
        if (in_array(strtoupper($resultMessage['state']), ['DELIVRD', 'DELIVERED']))
        {
            $resultMessage['status'] = 'DELIVERED';
        } elseif ((in_array(strtoupper($resultMessage['state']), ['ESME_ROK', 'ENROUTE','ACCEPTD'])))
        {
            $resultMessage['status'] = 'SENT';
        } elseif (isset($resultMessage[$this->getMessageField()]) && $this->isStatusPartOfRejectedErrorCodes(strtoupper($resultMessage['state']))){
            $resultMessage['status'] = 'REJECTED';
        }
        elseif (isset($resultMessage[$this->getMessageField()]))
        {
            $resultMessage['status'] = 'FAILED';
        }
        elseif($resultMessage['status'] != "EXPIRED")
        {
            $resultMessage['status'] = 'REJECTED';
        }
        return $resultMessage;
    }

    /**
     * Receive callbacks from Provider
     *
     * @param $data
     * @return array
     * @throws CustomException
     */
    public function receiveCallback($data)
    {
        return $this->mapper($data);
    }

    /**
     * Set the priority
     *

     * @param array $requestData
     *
     * @return $this
     * @throws \Throwable
     */
    public function requestData($requestData)
    {
        $this->priority = $requestData['priority'];
        return parent::requestData($requestData);
    }

    /**
     * @return string
     */
    public function getMessageField()
    {
        return 'message_id';
    }

    /**
     * Check if Alaris DLR returns any error codes that match the ones on FailedReasonEnum constants
     * @param $status
     * @return bool
     */
    private function isStatusPartOfRejectedErrorCodes($status)
    {
        $error_types = FailedReasonEnum::getConstantValues();
        foreach($error_types as $error_type){
            if(strstr($status, $error_type) !== FALSE){
                return true;
            }
        }
        return false;
    }
}
